/* @(#)gsfrec.c, Dirk Haun, 30.12.1997
 *
 * written 1997 by Dirk Haun and placed in the public domain
 *
 * Bewut primitives Programm, das eine GEMscript-Aufnahme
 * per GS_MACRO (mit Nullpointer in msg[3/4]) erzwingt.
 *
 * 27.12.1997  dh  erste Version
 * 28.12.1997  dh  erzeugt Scripter-Scripts
 * 30.12.1997  dh  umgeschrieben zum GTP (Name des Kommunikations-
 *                 partners kann bei Programmstart bergeben werden)
 *
 */

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <aes.h>
#include <tos.h>
#include <ctype.h>
#include <string.h>
#include <gscript.h>

#ifndef FALSE
# define FALSE (0)
# define TRUE  (1)
#endif

#define GS_ID 0x4242

/* Default-Programmname :-) */
char progname[] = "CHATWIN ";

static int appl_pwrite (int apid, ...)
{
  int      i, msg[8];
  va_list  pp;

  va_start (pp, apid);
  for (i = 0; i < 8; i++)
  {
    msg[i] = va_arg (pp, int);
  }
  i = appl_write (apid, 16, msg);
  va_end (pp);

  return (i);
}

static FILE *open_script (char *name)
{
  char *cp, fname[40];
  FILE *fh;

  strcpy (fname, name);
  cp = strchr (fname, ' ');
  *cp = '\0';
  strcat (fname, ".sic");
  cp = fname;
  while (*cp)
  {
    *cp++ = tolower (*cp);
  }

  fh = fopen (fname, "w");
  if (fh)
  {
    cp = strchr (fname, '.');
    *cp = '\0';
    fprintf (fh, "if (address (\"%s\")) {\n", fname);
  }

  return (fh);
}

static void close_script (FILE *fh)
{
  fprintf (fh, "}\nelse {\n  printf (\"Applikation nicht gefunden.\\n\");\n}\n");
  fclose (fh);
}

static char *quote_backslash (char *line)
{
  int   count = 0;
  char *cp, *wp, *new_line;

  cp = strchr (line, '\\');
  if (cp)
  {
    while (cp)
    {
      count++;
      cp++;
      cp = strchr (cp, '\\');
    }
    new_line = malloc (strlen (line) + count + 1);
    if (new_line)
    {
      wp = new_line;
      cp = line;
      while (*cp)
      {
        if (*cp == '\\')
        {
          *wp++ = '\\';
        }
        *wp++ = *cp++;
      }
      *wp = '\0';
    }

    return (new_line);
  }

  return (line);
}

static void log_command (FILE *fh, char *cmd)
{
  int   empty = FALSE;
  int   par = 0;
  char *cp;

  if (*cmd)
  {
    cp = quote_backslash (cmd);
    if (cp)
    {
      fprintf (fh, "  %s (", cp);
      if (cp != cmd)
      {
        free (cp);
      }
    }
    else /* bei Speichermangel wenigstens ungequotet schreiben */
    {
      fprintf (fh, "  %s (", cmd);
    }
  }
  else
  {
    empty = TRUE;
    fprintf (fh, "  /* (leeres Kommando)", cmd);
  }
  cmd += strlen (cmd) + 1;

  while (*cmd)
  {
    par++;
    if (par > 1)
    {
      fprintf (fh, ", ");
    }
    if (*cmd == 0x01) /* leerer Parameter */
    {
      fprintf (fh, "\"\"");
    }
    else if ((*cmd >= 0x02) && (*cmd <= 0x06))
    {
      fprintf (fh, "/* Parameter #%d ignoriert */", (int) *cmd);
    }
    else
    {
      cp = quote_backslash (cmd);
      if (cp)
      {
        fprintf (fh, "\"%s\"", cp);
        if (cp != cmd)
        {
          free (cp);
        }
      }
      else /* bei Speichermangel wenigstens ungequotet schreiben */
      {
        fprintf (fh, "\"%s\"", cmd);
      }
    }
    cmd += strlen (cmd) + 1;
  }
  if (empty)
  {
    fprintf (fh, " */\n");
  }
  else
  {
    fprintf (fh, ");\n");
  }
}

int main (int argc, char *argv[])
{
  int      i, apid, gl_apid, msg[8];
  int      quit = FALSE, recording = FALSE;
  char    *cmd;
  FILE    *fh;
  GS_INFO *gs_info, *app_info;

  if (argc > 1)
  {
    memset (progname, ' ', 8);
    progname[8] = '\0';
    for (i = 0; (i < 8) && (argv[1][i]); i++)
    {
      progname[i] = toupper (argv[1][i]);
    }
  }

  gl_apid = appl_init ();
  apid = appl_find (progname);
  if (apid >= 0)
  {
    /* GS_INFO-Struktur allokieren und initalisieren */
    gs_info = (GS_INFO*) Mxalloc (sizeof (GS_INFO), 0x43);
    if (-32L == (long) gs_info)
    {
      gs_info = (GS_INFO*) Malloc (sizeof (GS_INFO));
    }
    if (gs_info)
    {
      gs_info->len = sizeof (GS_INFO);
      gs_info->version = 0x0100;
      gs_info->msgs = GS_MACRO;
      gs_info->ext = '.sic';
      appl_pwrite (apid, GS_REQUEST, gl_apid, 0, (long) gs_info, 0, 0, GS_ID);

      /* Hauptschleife */
      while (!quit)
      {
        evnt_mesag (msg);
        switch (msg[0])
        {
          case GS_REPLY:
          {
            if (msg[1] == apid) /* von unserer App? */
            {
              if (msg[6] == 0) /* bereit fr GEMscript? */
              {
                app_info = *(GS_INFO**) &msg[3];
                if (app_info->msgs & GSM_WRITE) /* App ist aufnahmefhig */
                {
                  fh = open_script (progname);
                  if (fh)
                  {
                    appl_pwrite (apid, GS_MACRO, gl_apid, 0, 0L, fh->Handle, 0, 0);
                    recording = TRUE;
                  }
                  else /* Aufnahme abbrechen */
                  {
                    appl_pwrite (apid, GS_MACRO, gl_apid, 0, 0L, -1, -1, 0);
                    appl_pwrite (apid, GS_QUIT, gl_apid, 0, 0, 0, 0, 0, msg[7]);
                    quit = TRUE;
                  }
                }
                else
                {
                  form_alert (1, "[2][Applikation ist nicht |aufnahmefhig. ][ Abbruch ]");
                  quit = 1;
                }
              }
              else
              {
                form_alert (1, "[2][Applikation lehnt |GS-Kommunikation ab. ][ Abbruch ]");
                quit = TRUE;
              }
            }
          }
          break;

          case GS_QUIT:
          {
            if (msg[1] == apid)
            {
              if (recording)
              {
                close_script (fh);
                recording = FALSE;
              }
              quit = TRUE;
            }
          }
          break;

          case GS_WRITE:
          {
            if (recording)
            {
              cmd = *(char**) &msg[3];
              log_command (fh, cmd);
              appl_pwrite (apid, GS_ACK, gl_apid, 0, msg[3], msg[4], 0L, GSACK_OK);
            }
          }
          break;

          case GS_CLOSEMACRO:
          {
            close_script (fh);
            recording = FALSE;
            quit = TRUE;
          }
          break;

          case AP_TERM:
          {
            if (recording)
            {
              appl_pwrite (apid, GS_CLOSEMACRO, gl_apid, 0, 0, 0, fh->Handle, 0, 0);
              appl_pwrite (apid, GS_QUIT, gl_apid, 0, 0, 0, 0, 0, GS_ID);
              close_script (fh);
            }
            quit = TRUE;
          }
          break;
        }
      }
      Mfree (gs_info);
    }
    else
    {
      form_alert (1, "[2][Kein Speicher mehr. ][ Abbruch ]");
    }
  }
  else
  {
    form_alert (1, "[2][Applikation nicht gefunden. ][ Abbruch ]");
  }
  appl_exit();

  return (0);
}
