/* @(#)gsrecord.c, Dirk Haun, 30.12.1997
 *
 * written 1997 by Dirk Haun and placed in the public domain
 *
 * Primitiver kleiner GEMscript-Recorder. Wartet nach dem
 * Programmstart auf den Beginn einer Aufnahme per GS_OPENMACRO.
 *
 * 30.12.1997  dh  erste Version (abgeleitet von gsfrec.c)
 *
 * Bugs:
 * - merkt sich nur die ID (msg[7]) aus dem jeweils letzten GS_REQUEST
 *
 */

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <aes.h>
#include <tos.h>
#include <ctype.h>
#include <string.h>
#include <gscript.h>

#ifndef FALSE
# define FALSE (0)
# define TRUE  (1)
#endif

static int appl_pwrite (int apid, ...)
{
  int      i, msg[8];
  va_list  pp;

  va_start (pp, apid);
  for (i = 0; i < 8; i++)
  {
    msg[i] = va_arg (pp, int);
  }
  i = appl_write (apid, 16, msg);
  va_end (pp);

  return (i);
}

static FILE *open_script (char *filename, int apid)
{
  char *cp, ap_name[16];
  int   dummy, id;
  FILE *fh;

  strcpy (ap_name, "UNKNOWN ");
  if (appl_search (0, ap_name, &dummy, &id))
  {
    do
    {
      if (id == apid)
      {
        break;
      }
    }
    while (appl_search (1, ap_name, &dummy, &id));
  }

  fh = fopen (filename, "w");
  if (fh)
  {
    cp = strchr (ap_name, ' ');
    if (cp)
    {
      *cp = '\0';
    }
    fprintf (fh, "if (address (\"%s\")) {\n", ap_name);
  }

  return (fh);
}

static void close_script (FILE *fh)
{
  fprintf (fh, "}\nelse {\n  printf (\"Applikation nicht gefunden.\\n\");\n}\n");
  fclose (fh);
}

static char *quote_backslash (char *line)
{
  int   count = 0;
  char *cp, *wp, *new_line;

  cp = strchr (line, '\\');
  if (cp)
  {
    while (cp)
    {
      count++;
      cp++;
      cp = strchr (cp, '\\');
    }
    new_line = malloc (strlen (line) + count + 1);
    if (new_line)
    {
      wp = new_line;
      cp = line;
      while (*cp)
      {
        if (*cp == '\\')
        {
          *wp++ = '\\';
        }
        *wp++ = *cp++;
      }
      *wp = '\0';
    }

    return (new_line);
  }

  return (line);
}

static void log_command (FILE *fh, char *cmd)
{
  int   empty = FALSE;
  int   par = 0;
  char *cp;

  if (*cmd)
  {
    cp = quote_backslash (cmd);
    if (cp)
    {
      fprintf (fh, "  %s (", cp);
      if (cp != cmd)
      {
        free (cp);
      }
    }
    else /* bei Speichermangel wenigstens ungequotet schreiben */
    {
      fprintf (fh, "  %s (", cmd);
    }
  }
  else
  {
    empty = TRUE;
    fprintf (fh, "  /* (leeres Kommando)", cmd);
  }
  cmd += strlen (cmd) + 1;

  while (*cmd)
  {
    par++;
    if (par > 1)
    {
      fprintf (fh, ", ");
    }
    if (*cmd == 0x01) /* leerer Parameter */
    {
      fprintf (fh, "\"\"");
    }
    else if ((*cmd >= 0x02) && (*cmd <= 0x06))
    {
      fprintf (fh, "/* Parameter #%d ignoriert */", (int) *cmd);
    }
    else
    {
      cp = quote_backslash (cmd);
      if (cp)
      {
        fprintf (fh, "\"%s\"", cp);
        if (cp != cmd)
        {
          free (cp);
        }
      }
      else /* bei Speichermangel wenigstens ungequotet schreiben */
      {
        fprintf (fh, "\"%s\"", cmd);
      }
    }
    cmd += strlen (cmd) + 1;
  }
  if (empty)
  {
    fprintf (fh, " */\n");
  }
  else
  {
    fprintf (fh, ");\n");
  }
}

int main (void)
{
  int      gl_apid, gs_id, msg[8];
  int      apid = -1;
  int      quit = FALSE, recording = FALSE;
  char    *cmd, *filename;
  FILE    *fh = 0L;
  GS_INFO *gs_info;

  gl_apid = appl_init ();

  /* GS_INFO-Struktur allokieren und initalisieren */
  gs_info = (GS_INFO*) Mxalloc (sizeof (GS_INFO), 0x43);
  if (-32L == (long) gs_info)
  {
    gs_info = (GS_INFO*) Malloc (sizeof (GS_INFO));
  }
  if (gs_info)
  {
    gs_info->len = sizeof (GS_INFO);
    gs_info->version = 0x0100;
    gs_info->msgs = GSM_MACRO;
    gs_info->ext = '.sic';

    /* Hauptschleife */
    while (!quit)
    {
      evnt_mesag (msg);
      switch (msg[0])
      {
        case GS_REQUEST:
        {
          appl_pwrite (msg[1], GS_REPLY, gl_apid, 0, (long) gs_info, 0, 0, msg[7]);
          gs_id = msg[7];
        }
        break;

        case GS_QUIT:
        {
          if (msg[1] == apid)
          {
            if (recording)
            {
              close_script (fh);
              recording = FALSE;
            }
            quit = TRUE;
          }
        }
        break;

        case GS_OPENMACRO:
        {
          if (recording)
          {
            /* wir knnen nur eine Aufnahme gleichzeitig: Ablehnung */
            appl_pwrite (msg[1], GS_MACRO, gl_apid, 0, msg[3], msg[4], -1, -1, 0);
          }
          else
          {
            filename = *(char**) &msg[3];
            fh = open_script (filename, msg[1]);
            if (fh)
            {
              appl_pwrite (msg[1], GS_MACRO, gl_apid, 0, msg[3], msg[4], fh->Handle, 0, 0);
              apid = msg[1];
              recording = TRUE;
            }
            else /* Fehler: Aufnahme abbrechen */
            {
              appl_pwrite (msg[1], GS_MACRO, gl_apid, 0, msg[3], msg[4], -1, -1, 0);
              appl_pwrite (msg[1], GS_QUIT, gl_apid, 0, 0, 0, 0, 0, gs_id);
              quit = TRUE;
            }
          }
        }
        break;

        case GS_WRITE:
        {
          if (recording)
          {
            cmd = *(char**) &msg[3];
            log_command (fh, cmd);
            appl_pwrite (apid, GS_ACK, gl_apid, 0, msg[3], msg[4], 0L, GSACK_OK);
          }
        }
        break;

        case GS_CLOSEMACRO:
        {
          if (msg[1] == apid)
          {
            close_script (fh);
            recording = FALSE;
            quit = TRUE;
          }
        }
        break;

        case AP_TERM:
        {
          if (recording)
          {
            appl_pwrite (apid, GS_CLOSEMACRO, gl_apid, 0, 0, 0, fh->Handle, 0, 0);
            appl_pwrite (apid, GS_QUIT, gl_apid, 0, 0, 0, 0, 0, gs_id);
            close_script (fh);
          }
          quit = TRUE;
        }
        break;
      }
    }
    Mfree (gs_info);
  }
  else
  {
    form_alert (1, "[2][Kein Speicher mehr. ][ Abbruch ]");
  }
  appl_exit();

  return (0);
}
